--
-- YawningCombat.lua: Yawning's Combat Timer
-- Yawning (Crushridge-US)
--

YawningCombat = LibStub("AceAddon-3.0"):NewAddon("YawningCombat", "AceEvent-3.0");
local AceConfig = LibStub("AceConfig-3.0");
local AceConfigDialog = LibStub("AceConfigDialog-3.0");

--
-- Anchor Management.
--

local function onDragStart(self) self:StartMoving(); end
local function onDragStop(self)
	self:StopMovingOrSizing();

	local s = self:GetEffectiveScale();
	YawningCombat.db.char.xpos = self:GetLeft() * s;
	YawningCombat.db.char.ypos = self:GetTop() * s;
end

function YawningCombat:CreateAnchor()
	--
	-- Anchor creation code swiped from BigWigs.
	--

	local anchor = CreateFrame("Frame", "YawningCombat Anchor", UIParent);
	local s = anchor:GetEffectiveScale();

	anchor:EnableMouse(true);
	anchor:SetMovable(true);
	anchor:RegisterForDrag("LeftButton");
	anchor:SetScript("OnDragStart", onDragStart);
	anchor:SetScript("OnDragStop", onDragStop);
	anchor:ClearAllPoints();

	if ((self.db.char.xpos == -1) or (self.db.char.ypos == -1)) then
		anchor:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
	else
		anchor:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.char.xpos / s, self.db.char.ypos / s);
	end

	anchor:SetWidth(self.db.char.width);
	anchor:SetHeight(self.db.char.height);

	local bg = anchor:CreateTexture(nil, "PARENT");
	bg:SetAllPoints(anchor);
	bg:SetBlendMode("BLEND");
	bg:SetTexture(0, 0, 0, 0.3);

	local header = anchor:CreateFontString(nil, "OVERLAY");
	header:SetFontObject(GameFontNormal);
	header:SetText("OoC");
	header:SetAllPoints(anchor);
	header:SetJustifyH("CENTER");
	header:SetJustifyV("MIDDLE");

	if (self.db.char.visual.font ~= nil) then
		local _, sz, _ = header:GetFont();
		header:SetFont(self.SM:Fetch("font", self.db.char.visual.font), sz);
	end

	self.timerAnchor = anchor;
	self.displayText = header;

	self.timerAnchor:Hide();

	self:ToggleAnchor();
end

function YawningCombat:ToggleAnchor()
	if (self.db.char.locked == false) then
		self.timerAnchor:EnableMouse(true);
	else
		self.timerAnchor:EnableMouse(false);
	end
end

function YawningCombat:RefreshAnchor()
	if (self.db.char.locked == false) then
		self.timerAnchor:Show();
	elseif (self.db.char.hideLocked == false) then
		if (self:ShouldDisplayBar()) then
			self.timerAnchor:Show();
		else
			self.timerAnchor:Hide();
		end
	else
		self.timerAnchor:Hide();
	end
end

--
-- Timer Management functions.
--

function YawningCombat:Start()
	if (self.timer == nil) then
		self.timer = LibStub("LibCandyBar-3.0"):New(self.SM:Fetch("statusbar", self.db.char.visual.texture), 150, 25);
		self.timer.anchor = self.timerAnchor;
		self.timer:SetTimeVisibility(true);

		self:RefreshTimer();
	end

	self.timer:SetDuration(5 + self.db.char.latencyFudge); 
	self.timer:Start();

	--
	-- HACK HACK HACK
	-- Reach into the guts of LibCandyBar and change the text widget to center the duration
	-- on the bar.  I do this here instead of in RefreshTimer because the Start() call adjusts
	-- candyBarDuration. and it would trample over my changes.
	--

	self.timer.candyBarDuration:SetAllPoints(self.timer.candyBarBar);
	self.timer.candyBarDuration:SetJustifyH("CENTER");
end


function YawningCombat:RefreshTimer()
	if (self.timer ~= nil) then
		self.timer:SetColor(self.db.char.visual.r, self.db.char.visual.g, self.db.char.visual.b, self.db.char.visual.a);

		--
		-- As much as reaching into the internal representation of the bar is lame, 
		--
		if (self.db.char.visual.font ~= nil) then
			local _, sz, _ = self.timer.candyBarDuration:GetFont();
			self.timer.candyBarDuration:SetFont(self.SM:Fetch("font", self.db.char.visual.font), sz);
		end

		self.timer:SetTexture(self.SM:Fetch("statusbar", self.db.char.visual.texture));
		self.timer.candyBarBar:GetStatusBarTexture():SetHorizTile(false);

		self.timer:ClearAllPoints();
		self.timer:SetAllPoints(self.timerAnchor);
	end
end

function YawningCombat:Stop()
	if (self.timer ~= nil) then
		self.timer:Stop();
	end

	self.timer = nil;
end

function YawningCombat:Reset()
	self:Start();
end

--
-- PLAYER_ENTERING_WORLD
--

function YawningCombat:ShouldDisplayBar()
	local isInstance, instanceType = IsInInstance();

	if (self.db.char.zone[instanceType] ~= nil) then
		if (self.db.char.zone[instanceType] == true) then
			return true;
		end
	end

	return false;
end

function YawningCombat:PLAYER_ENTERING_WORLD()
	self:RefreshAnchor();

	if (self:ShouldDisplayBar()) then
		self:RegisterEvent("PLAYER_REGEN_ENABLED");
		self:RegisterEvent("PLAYER_REGEN_DISABLED");

		if (InCombatLockdown()) then
			self:PLAYER_REGEN_DISABLED();
		end
	else
		self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");

		self:UnregisterEvent("PLAYER_REGEN_ENABLED");
		self:UnregisterEvent("PLAYER_REGEN_DISABLED");

		self:Stop();
	end
end

--
-- PLAYER_REGEN_ENABLED
--

function YawningCombat:PLAYER_REGEN_ENABLED()
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");

	self.displayText:SetText("OoC");

	-- Stop the timer
	self:Stop();
end

--
-- PLAYER_REGEN_DISABLED
--

function YawningCombat:PLAYER_REGEN_DISABLED()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");

	self.displayText:SetText("");

	-- Start the timer
	self:Start();
end

--
-- COMBAT_LOG_EVENT_UNFILTERED
--
-- This is shamelessly copied from:
-- http://www.wowinterface.com/downloads/info8493-CombatTimer.html
--

local eventRegistered = {
	["SWING_DAMAGE"] = true, --someone got damaged by meele
	["SWING_MISSED"] = true, --someone missed, resisted, absorbed, etc. damage by meele
	["RANGE_DAMAGE"] = true, --someone got damaged by range
	["RANGE_MISSED"] = true, --someone missed, resisted, absorbed, etc. damage by range
	["SPELL_DAMAGE"] = true, --someone got damaged by caster 
	["SPELL_MISSED"] = true, --someone missed, resisted, absorbed, etc. damage by caster
	["SPELL_HEAL"] = true, --someone got healed by caster
	["SPELL_CAST_SUCCESS"] = true, --some got affected by instant spell like Counterspell
	["SPELL_AURA_APPLIED"] = true, --someone got buffed/debuffed by caster
	["SPELL_AURA_DISPELLED"] = true, --someones buff/debuff got dispelled by caster
	["SPELL_AURA_STOLEN"] = true, --someones buff got stolen by caster
	["SPELL_DISPEL_FAILED"] = true, --caster failed to dispel buff/debuff
	["SPELL_PERIODIC_DISPEL_FAILED"] = true, --caster failed to dispel dot/hot
};

local COMBATLOG_OBJECT_AFFILIATION_MINE = COMBATLOG_OBJECT_AFFILIATION_MINE;
local COMBATLOG_OBJECT_REACTION_FRIENDLY = COMBATLOG_OBJECT_REACTION_FRIENDLY;

function YawningCombat:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, eventType, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, ...)
	--return if we don't track that event
	if( not eventRegistered[eventType] ) then
		return;
	end
	
	local isDestPlayer = (bit.band(destFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) == COMBATLOG_OBJECT_AFFILIATION_MINE);
	local isSourcePlayer = (bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) == COMBATLOG_OBJECT_AFFILIATION_MINE);
	local isSourceFriend = (bit.band(destFlags, COMBATLOG_OBJECT_REACTION_FRIENDLY) == COMBATLOG_OBJECT_REACTION_FRIENDLY);

	--return if event dest or source is not player
	if( not isDestPlayer and not isSourcePlayer ) then
		return;
	end
	
	--return if player only gets healed or buffed by someone
	if( isDestPlayer and (eventType == "SPELL_HEAL" or (eventType == "SPELL_AURA_APPLIED" and isSourceFriend)) ) then
		return;
	end

	--return if the event is listed in our quirk table
	if ((spellId ~= nil) and (self.Quirks[spellId])) then
		return;
	end

	--reset timer because player participated in combat
	self:Reset();
end

--
-- Initialization function.
--

function YawningCombat:OnInitialize()
	self.timer = nil;
	self.timerAnchor = nil;
	self.displayText = nil;

	self.SM = LibStub("LibSharedMedia-3.0", true);
	self:LoadDefaults();
	self:LoadOptions();

	self.db = LibStub("AceDB-3.0"):New("YawningCombatDB", self.defaults, "char");
	self.db:RegisterDefaults(self.defaults);
	self.optFrame = AceConfigDialog:AddToBlizOptions("YawningCombat", "YawningCombat");
	AceConfig:RegisterOptionsTable("YawningCombat", self.options);

	self:CreateAnchor();

	if (self.db.char.isEnabled == false) then
		self:Disable();
	end
end

--
-- User Enable.
--

function YawningCombat:OnEnable()
	self.db.char.isEnabled = true;

	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "PLAYER_ENTERING_WORLD");
end

--
-- User Disable.
--

function YawningCombat:OnDisable()
	self.db.char.isEnabled = false;

	self:Stop();
	self.timerAnchor:Hide();

	self:UnregisterAllEvents();
end

