--
-- YawningCombat_Options.lua: Yawning's Combat Timer Options
-- Yawning (Crushridge-US)
--

--
-- YawningCombat is defined in the main file.
--

if not YawningCombat then return end

--
-- Load the defaults
--

function YawningCombat:LoadDefaults()
	self.defaults = {
		char = {
			isEnabled = true,
			locked = true,
			hideLocked = false,
			width = 150,
			height = 25,
			xpos = -1,
			ypos = -1,
			latencyFudge = 0,
			zone = {
				none = false,
				pvp = true,
				arena = true,
				party = false,
				raid = false,
			},
			visual = {
				r = 0.15,
				g = 1.0,
				b = 0.0,
				a = 1.0,
				texture = "Blizzard",
				font = "Friz Quadrata TT",
			},
		},
	};
end

--
-- Configuration GUI
--

function YawningCombat:LoadOptions()

	local behaviorOptions = {
		name = "Behavior",
		type = "group",
		inline = true,
		order = 0,
		args = {
			enable = {
				name = "Enable",
				type = "toggle",
				order = 0,
				set = function(info, val) YawningCombat:OnEnableSet(val) end,
				get = function(info) return YawningCombat:IsEnabled() end,
			},
			hideLocked = {
				name = "Hide when OoC",
				type = "toggle",
				order = 1,
				set = function(info, val) self:OnToggleHide(val) end,
				get = function(info) return self.db.char.hideLocked; end,
			},
			latencyFudge = {
				name = "Extra Time",
				type = "range",
				min = 0,
				max = 1,
				step = 0.1,
				set = function(info, val) self.db.char.latencyFudge = val end,
				get = function(info) return self.db.char.latencyFudge; end,
			}
		},
	};

	local barOptions = {
		name = "Layout",
		type = "group",
		inline = true,
		order = 1,
		args = {
			lock = {
				name = "Lock",
				type = "toggle",
				order = 0,
				set = function(info, val) self:OnLockSet(val) end,
				get = function(info) return self.db.char.locked; end,
			},
			width = {
				name = "Width",
				type = "range",
				min = 50,
				max = 250,
				step = 1,
				bigStep = 5,
				order = 1,
				set = function(info, val) self:OnWidthSet(val) end,
				get = function(info) return self.db.char.width; end,
			},
			height = {
				name = "Height",
				type = "range",
				min = 10,
				step = 1,
				bigStep = 5,
				order = 2,
				set = function(info, val) self:OnHeightSet(val) end,
				get = function(info) return self.db.char.height; end,
			},
		}
	};

	local appearanceOptions = {
		name = "Appearance",
		type = "group",
		inline = true,
		order = 2,
		args = {
			color = {
				name = "Color",
				type = "color",
				hasAlpha = true,
				order = 1,
				set = function(info, r, g, b, a) self:OnColorSet(r, g, b, a) end,
				get = function(info) return self.db.char.visual.r, self.db.char.visual.g, self.db.char.visual.b, self.db.char.visual.a; end,
			},
			texture = {
				name = "Texture",
				order = 2,
				type = "select",
				dialogControl = "LSM30_Statusbar",
				values = self.SM:HashTable("statusbar"),
				set = function(info, val) self:OnTextureSet(val) end,
				get = function(info) return self.db.char.visual.texture; end,
			},
			font = {
				name = "Font",
				order = 3,
				type = "select",
				values = self.SM:HashTable("font");
				dialogControl = "LSM30_Font",
				set = function(info, val) self:OnFontSet(val) end,
				get = function(info) return self.db.char.visual.font; end,
			},
		},
	};

	local zoneOptions = {
		name = "Enable Inside",
		type = "group",
		inline = true,
		order = 3,
		args = {
			arenas = {
				name = "Arenas",
				type = "toggle",
				set = function(info, val) self.db.char.zone.arena = val; self:OnZoneSet(); end,
				get = function(info) return self.db.char.zone.arena end,
			},
			everywhere = {
				name = "Everywhere else",
				type = "toggle",
				set = function(info, val) self.db.char.zone.none = val; self:OnZoneSet(); end,
				get = function(info) return self.db.char.zone.none end,
			},
			party = {
				name = "Party instances",
				type = "toggle",
				set = function(info, val) self.db.char.zone.party = val; self:OnZoneSet(); end,
				get = function(info) return self.db.char.zone.party end,
			},
			bg = {
				name = "Battlegrounds",
				type = "toggle",
				set = function(info, val) self.db.char.zone.pvp = val; self:OnZoneSet(); end,
				get = function(info) return self.db.char.zone.pvp end,
			},
			raid = {
				name = "Raid instances",
				type ="toggle",
				set = function(info, val) self.db.char.zone.raid = val; self:OnZoneSet(); end,
				get = function(info) return self.db.char.zone.raid end,
			}
		},
	};

	self.options = {
		name = "YawningCombat",
		type = "group",
		args = {
			behaviorOptions = behaviorOptions,
			barOptions = barOptions,
			appearanceOptions = appearanceOptions,
			zoneOptions = zoneOptions,
		},
	};
end

--
-- Workers
--

function YawningCombat:OnEnableSet(val)
	if (val == true) then
		self:Enable();
	else
		self:Disable();
	end
end

function YawningCombat:OnWidthSet(val)
	self.db.char.width = val;

	self.timerAnchor:SetWidth(self.db.char.width);
	self:RefreshTimer();
end

function YawningCombat:OnHeightSet(val)
	self.db.char.height = val;

	self.timerAnchor:SetHeight(self.db.char.height);
	self:RefreshTimer();
end

function YawningCombat:OnLockSet(val)
	self.db.char.locked = val;

	self:ToggleAnchor();
	self:RefreshAnchor();
end

function YawningCombat:OnZoneSet()
	if (self.db.char.isEnabled == true) then
		self:PLAYER_ENTERING_WORLD();
	end
end

function YawningCombat:OnToggleHide(val)
	self.db.char.hideLocked = val;

	self:RefreshAnchor();
end

function YawningCombat:OnColorSet(r, g, b, a)
	self.db.char.visual.r = r;
	self.db.char.visual.g = g;
	self.db.char.visual.b = b;
	self.db.char.visual.a = a;

	self:RefreshTimer();
end

function YawningCombat:OnTextureSet(val)
	self.db.char.visual.texture = val;

	self:RefreshTimer();
end

function YawningCombat:OnFontSet(val)
	local _, sz, _ = self.displayText:GetFont();

	self.db.char.visual.font = val;
	self.displayText:SetFont(self.SM:Fetch("font", val), sz);

	self:RefreshTimer();
end

